import yaml
from typing import Literal, Any, Optional, Mapping
from theia import Theia
from theia.aws.network.vpc import Vpc, VpcSettings

with open ('input.yaml','r') as f:
    defaultInputs = yaml.safe_load(f)

def getProject(userinputs: dict):
    return f"{userinputs['project']}-{userinputs['env']}"

class testbase:

    @staticmethod
    def getBaseResourceName(userInputs: dict):
        resourceList = {}
        baseStack = f"{userInputs['project']}-{userInputs['env']}-{testbase.__name__}"
        vpc = {"vpc1" : f"{baseStack}-vpc"}
        resourceList.update(vpc)
        return resourceList
    
    def getStackName(self):
        stack = f"{self.userInputs['project']}-{self.userInputs['env']}-{self.stack}"
        return stack
    
    def __init__(self,
                 userInputs: dict,
                 defaultInputs: Optional[dict] = defaultInputs):
        
        self.userInputs = userInputs
        self.defaultInputs = defaultInputs

        self.stack = testbase.__name__
        
        self.project = getProject(self.userInputs)
        self.region = self.userInputs['region']
        self.email = self.userInputs['email']
        self.backend_url = self.userInputs['backend_url']
        self.vpc_ipv4_cidr = self.userInputs['ipv4_cidr']

        self.defaultInputs['tags']['created_by'] = f"{self.email}"
        resourceTags = {"Stack" : f"{self.getStackName()}"}
        self.defaultInputs['tags'].update(resourceTags)
        self.defaultInputs['tags'].update(self.userInputs['tags'])
        self.tags = self.defaultInputs['tags']


    def infra(self):

        resourceList = testbase.getBaseResourceName(self.userInputs)

        def program():
            vpcName = resourceList['vpc1']
            vpcCidr = self.vpc_ipv4_cidr
            vpcTags = self.tags

            vpc = Vpc(resource_name=vpcName,
                      ipv4_cidr=vpcCidr,
                      tenancy="default",
                      settings=VpcSettings(),
                      tags=vpcTags)
            
        t =  Theia(project=self.project,
                    stack=self.stack,
                    program=program,
                    region=self.region,
                    backend_url=self.backend_url)
            
        return t
        
    def __call__(self, command: Literal['preview', 'up', 'destroy', 'refresh']) -> Any:
        t = self.infra()
        getattr(t, command)()

# userInputs={
#     "project" : "test",
#     "env" : "dev",
#     "region" : "us-east-1",
#     "email" : "ayush.kumar@amagi.com",
#     "backend_url" : "s3://pcp-as-code-draft-ayush",
#     "ipv4_cidr" : "10.0.0.0/16",
#     "tags" : {
#     }
# }

# t = testbase(userInputs={
#     "project" : "test",
#     "env" : "dev",
#     "region" : "us-east-1",
#     "email" : "ayush.kumar@amagi.com",
#     "backend_url" : "s3://pcp-as-code-draft-ayush",
#     "ipv4_cidr" : "10.0.0.0/16",
#     "tags" : {

#     }
# })

# t('preview')